# -*- coding: utf-8 -*-
# Copyright (c) 2025-2025 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Optional

import modin.pandas as pd
from loguru import logger
from pandas import DataFrame

from msserviceprofiler.modelevalstate.data_feature.dataset_with_swifter import MyDataSetWithSwifter


class MyDataSetWithModin(MyDataSetWithSwifter):

    def preprocess_dispatch(self, lines_data: Optional[DataFrame] = None):
        logger.debug(f"start construct_data with modin, shape {lines_data.shape}")
        try:
            return self.preprocess(pd.DataFrame(lines_data))
        except Exception as e:
            logger.error(f"Failed in construct data with modin. error: {e}")
            return super(MyDataSetWithModin, self).preprocess_dispatch(lines_data)
